CREATE DATABASE meca with log;

DATABASE meca;

GRANT CONNECT TO admin;

CREATE TABLE AccessControlList
      (loginName                varchar(8) NOT NULL,
       groupName                varchar(8) NOT NULL,
       subscribedFlag           character
);

CREATE UNIQUE CLUSTER INDEX XPKAccessControlLi
  ON AccessControlList
(
       loginName                      ASC,
       groupName                      ASC
);


ALTER TABLE AccessControlList
ADD CONSTRAINT  PRIMARY KEY (loginName, groupName);


CREATE TABLE Events
      (eventId                  serial NOT NULL,
       loginName                varchar(8) NOT NULL,
       eventGMTTime             varchar(40),
       eventStartDate           date,
       eventStopDate            date,
       shortDescription         varchar(150),
       longDescription          varchar(255),
       eventURL                 varchar(100)
);

CREATE UNIQUE CLUSTER INDEX XPKEvents
  ON Events
(
       eventId                        ASC,
       loginName                      ASC
);


ALTER TABLE Events
ADD CONSTRAINT  PRIMARY KEY (eventId, loginName);


CREATE TABLE MecaGroup
      (loginName                varchar(8) NOT NULL,
       groupType                character,
       loginPassword            varchar(10),
       groupOwner               varchar(8),
       groupDescription         varchar(25),
       groupPermission          character
);

CREATE UNIQUE CLUSTER INDEX XPKGroup
  ON MecaGroup
(
       loginName                      ASC
);


ALTER TABLE MecaGroup
ADD CONSTRAINT  PRIMARY KEY (loginName);


ALTER TABLE AccessControlList
ADD CONSTRAINT  FOREIGN KEY  (loginName)
               REFERENCES MecaGroup;


ALTER TABLE Events
ADD CONSTRAINT  FOREIGN KEY  (loginName)
               REFERENCES MecaGroup;

create procedure erwin_raise_except(err int,msg varchar(255))
  raise exception err,0,msg;
end procedure;

create trigger tI_AccessControlLi INSERT on AccessControlList
  referencing NEW as inserted
  for each row
-- ERwin Builtin Mon Aug 19 19:55:40 1996
-- INSERT trigger on AccessControlList
    -- ERwin Builtin Mon Aug 19 19:55:40 1996
    -- MecaGroup R/2 AccessControlList ON CHILD INSERT RESTRICT 
    when (
      -- %NotnullFK(inserted," is not null and") 
      
      not exists (
        select * from MecaGroup
        where
          -- %JoinFKPK(inserted,MecaGroup," = "," and") 
          inserted.loginName = MecaGroup.loginName
      )
    )
      (execute procedure erwin_raise_except(
        -746,
        'Cannot INSERT "AccessControlList" because "MecaGroup" does not exist.'
      ))
;

create trigger tU_AccessControlLi UPDATE on AccessControlList
  referencing NEW as inserted OLD as deleted
  for each row
-- ERwin Builtin Mon Aug 19 19:55:40 1996
-- UPDATE trigger on AccessControlList
    -- ERwin Builtin Mon Aug 19 19:55:40 1996
    -- MecaGroup R/2 AccessControlList ON CHILD UPDATE RESTRICT 
    when (
      -- %NotnullFK(inserted," is not null and") 
      
      not exists (
        select * from MecaGroup
        where
          -- %JoinFKPK(inserted,MecaGroup," = "," and") 
          inserted.loginName = MecaGroup.loginName
      )
    )
      (execute procedure erwin_raise_except(
        -746,
        'Cannot UPDATE "AccessControlList" because "MecaGroup" does not exist.'
      ))
;

create trigger tI_Events INSERT on Events
  referencing NEW as inserted
  for each row
-- ERwin Builtin Mon Aug 19 19:55:40 1996
-- INSERT trigger on Events
    -- ERwin Builtin Mon Aug 19 19:55:40 1996
    -- MecaGroup R/1 Events ON CHILD INSERT RESTRICT 
    when (
      -- %NotnullFK(inserted," is not null and") 
      
      not exists (
        select * from MecaGroup
        where
          -- %JoinFKPK(inserted,MecaGroup," = "," and") 
          inserted.loginName = MecaGroup.loginName
      )
    )
      (execute procedure erwin_raise_except(
        -746,
        'Cannot INSERT "Events" because "MecaGroup" does not exist.'
      ))
;

create trigger tU_Events UPDATE on Events
  referencing NEW as inserted OLD as deleted
  for each row
-- ERwin Builtin Mon Aug 19 19:55:40 1996
-- UPDATE trigger on Events
    -- ERwin Builtin Mon Aug 19 19:55:40 1996
    -- MecaGroup R/1 Events ON CHILD UPDATE RESTRICT 
    when (
      -- %NotnullFK(inserted," is not null and") 
      
      not exists (
        select * from MecaGroup
        where
          -- %JoinFKPK(inserted,MecaGroup," = "," and") 
          inserted.loginName = MecaGroup.loginName
      )
    )
      (execute procedure erwin_raise_except(
        -746,
        'Cannot UPDATE "Events" because "MecaGroup" does not exist.'
      ))
;

create trigger tD_MecaGroup DELETE on MecaGroup
  referencing OLD as deleted
  for each row
-- ERwin Builtin Mon Aug 19 19:55:40 1996
-- DELETE trigger on MecaGroup
    -- ERwin Builtin Mon Aug 19 19:55:40 1996
    -- MecaGroup R/2 AccessControlList ON PARENT DELETE RESTRICT 
    when (exists (
      select * from AccessControlList
      where
        --  %JoinFKPK(AccessControlList,deleted," = "," and") 
        AccessControlList.loginName = deleted.loginName
    ))
       (execute procedure erwin_raise_except(
         -746,
         'Cannot DELETE "MecaGroup" because "AccessControlList" exists.'
       )),
    -- ERwin Builtin Mon Aug 19 19:55:40 1996
    -- MecaGroup R/1 Events ON PARENT DELETE RESTRICT 
    when (exists (
      select * from Events
      where
        --  %JoinFKPK(Events,deleted," = "," and") 
        Events.loginName = deleted.loginName
    ))
       (execute procedure erwin_raise_except(
         -746,
         'Cannot DELETE "MecaGroup" because "Events" exists.'
       ))
;

create trigger tU_MecaGroup UPDATE on MecaGroup
  referencing NEW as inserted OLD as deleted
  for each row
-- ERwin Builtin Mon Aug 19 19:55:40 1996
-- UPDATE trigger on MecaGroup
  -- ERwin Builtin Mon Aug 19 19:55:40 1996
  -- MecaGroup R/2 AccessControlList ON PARENT UPDATE RESTRICT 
  when(
    not (
        -- %JoinPKPK(inserted,deleted," = "," and") 
        inserted.loginName = deleted.loginName
    ) and
    exists (
      select * from AccessControlList
      where
        --  %JoinFKPK(AccessControlList,deleted," = "," and") 
        AccessControlList.loginName = deleted.loginName
    ))
       (execute procedure erwin_raise_except(
         -746,
         'Cannot UPDATE "MecaGroup" because "AccessControlList" exists.'
       )),
  -- ERwin Builtin Mon Aug 19 19:55:40 1996
  -- MecaGroup R/1 Events ON PARENT UPDATE RESTRICT 
  when(
    not (
        -- %JoinPKPK(inserted,deleted," = "," and") 
        inserted.loginName = deleted.loginName
    ) and
    exists (
      select * from Events
      where
        --  %JoinFKPK(Events,deleted," = "," and") 
        Events.loginName = deleted.loginName
    ))
       (execute procedure erwin_raise_except(
         -746,
         'Cannot UPDATE "MecaGroup" because "Events" exists.'
       ))
;

